import { unique } from 'shorthash';
export { default as shorthash } from 'shorthash';
import { declare } from '@babel/helper-plugin-utils';
import { types, transformFileAsync } from '@babel/core';
import { codeFrameColumns } from '@babel/code-frame';
import glob from 'glob';
import { join, isAbsolute, dirname, basename, extname } from 'path';
import { sync } from 'mkdirp';
import { existsSync, readFileSync, writeFileSync, promises } from 'fs';
import { promisify } from 'util';
import Excel from 'exceljs';
import { exec } from 'child_process';

var genId = function genId(message, scope) {
  return unique(message + (scope || ''));
};

var cloneJSXOpeningElement = function cloneJSXOpeningElement(path, descriptor, msgAttr, removeDefaultMessage) {
  return types.jsxOpeningElement(types.jsxIdentifier(path.node.name.name), [].concat(removeDefaultMessage ? path.node.attributes.filter(function (attr) {
    return attr !== msgAttr;
  }) : path.node.attributes, [types.jSXAttribute(types.jsxIdentifier('id'), types.stringLiteral(descriptor.id))]));
};
var cloneNormalCallExpression = function cloneNormalCallExpression(path, name, descriptor, msgProp, removeDefaultMessage) {
  var node = path.node;
  var args = node.arguments[0];
  var pathArgs = [types.objectExpression([].concat(removeDefaultMessage ? args.properties.filter(function (p) {
    return p !== msgProp;
  }) : args.properties, [types.objectProperty(types.identifier('id'), types.stringLiteral(descriptor.id))]))].concat(path.node.arguments.slice(1));

  if (types.isIdentifier(node.callee)) {
    return types.callExpression(types.identifier(name), pathArgs);
  }

  var memberCallee = node.callee;
  var object = memberCallee.object;
  return types.callExpression(types.memberExpression(types.identifier(object.name), types.identifier(memberCallee.property.name)), pathArgs);
};

var scopeRegexp = /@message\.scope(?:[\t\x20])+([\w-]+)/;

var praseComments = function praseComments(leadingComments) {
  var _leadingComments$map$;

  if (!leadingComments) {
    return null;
  }

  var scope = (_leadingComments$map$ = leadingComments.map(function (item) {
    return item.value;
  }).map(function (item) {
    var matched = item.match(scopeRegexp);
    return matched == null ? void 0 : matched[1];
  }).filter(Boolean)) == null ? void 0 : _leadingComments$map$[0];
  var matchedList = [];
  leadingComments.forEach(function (_ref) {
    var value = _ref.value;
    var data = value.split(/\n/);
    data.forEach(function (item) {
      var tmp;
      var messagesRegexp = /@message\.([a-z]{2}-[A-Z]{2})(?:[\t\x20])+([^\s@]+(?:\s+?[^\s@]+)*)/g;

      while (tmp = messagesRegexp.exec(item)) {
        matchedList.push(tmp);
      }
    });
  });
  var data = matchedList.reduce(function (result, item) {
    var locale = item[1],
        message = item[2];

    if (!locale || !message) {
      return result;
    }

    result[locale] = message;
    return result;
  }, {});
  return {
    scope: scope,
    messages: Object.keys(data).length ? data : undefined
  };
};

var ShortcutFunctionName = 'formatMessage';
var CalleeFunctionNames = ['formatMessage'];
var ComponentNames = ['FormattedMessage'];
var ScanConfigFilename = 'mes-babel-plugin-intl.config.js';
var UmiPluginKey = 'mes-babel-plugin-intl';
var I18nMessageFilename = './i18n-messages/data.json';
var LocaleMap = {
  zh: 'zh-CN',
  en: 'en-US'
};
var DistributionName = 'Distribution';
var ExcelFilename = 'translated.xlsx';
var InternalTranslatedDataFilename = 'internal-translated.js';

function createJSXMessageDescriptor(path, writeMode, removeDefaultMessage) {
  if (writeMode === void 0) {
    writeMode = true;
  }

  if (removeDefaultMessage === void 0) {
    removeDefaultMessage = false;
  }

  var attributes = path.node.attributes;
  var descriptor = {};
  var msgAttr = null;
  attributes.forEach(function (attr) {
    if (types.isJSXAttribute(attr) && types.isJSXIdentifier(attr.name) && types.isStringLiteral(attr.value)) {
      var _attr$value$value;

      var value = (_attr$value$value = attr.value.value) == null ? void 0 : _attr$value$value.trim();
      var name = attr.name.name;

      if (name === 'id') {
        descriptor.id = value;
      } else if (name === 'defaultMessage') {
        msgAttr = attr;
        descriptor.defaultMessage = value;
      }
    }
  });

  if (descriptor.defaultMessage) {
    if (!descriptor.id) {
      descriptor.id = genId(descriptor.defaultMessage);

      if (writeMode) {
        // 原地加上 id
        path.replaceWith(cloneJSXOpeningElement(path, descriptor, msgAttr, removeDefaultMessage));
        path.skip();
      }
    } else if (removeDefaultMessage) {
      path.replaceWith(cloneJSXOpeningElement(path, descriptor, msgAttr, true));
      path.skip();
    }

    return descriptor;
  }

  return null;
}

function createFunctionMessageDescriptor(path, shortcut, writeMode, removeDefaultMessage) {
  if (writeMode === void 0) {
    writeMode = true;
  }

  if (removeDefaultMessage === void 0) {
    removeDefaultMessage = false;
  }

  var node = path.node;
  var name = types.isIdentifier(node.callee) ? node.callee.name : node.callee.property.name;
  var args = node.arguments[0];
  var restArgs = node.arguments[1];
  var maybeIdArgs = node.arguments[2];
  var descriptor = {};
  var msgProp = null;
  var hasQuery = false;
  var translatedMessage = praseComments(node.leadingComments);

  if (types.isIdentifier(node.callee) && typeof shortcut === 'string' && name === shortcut && types.isStringLiteral(args) && args.value.trim()) {
    descriptor.defaultMessage = args.value.trim();

    if (types.isStringLiteral(restArgs) || types.isObjectExpression(restArgs) && restArgs.properties.length > 0) {
      hasQuery = types.isObjectExpression(restArgs) && restArgs.properties.length > 0;
      descriptor.id = types.isStringLiteral(restArgs) ? restArgs.value.trim() : types.isStringLiteral(maybeIdArgs) ? maybeIdArgs.value.trim() : '';
    }

    if (translatedMessage != null && translatedMessage.messages) {
      descriptor.translatedMessage = translatedMessage.messages;
    }

    if (!descriptor.id) {
      descriptor.id = genId(descriptor.defaultMessage, translatedMessage == null ? void 0 : translatedMessage.scope);

      if (writeMode) {
        path.replaceWith(types.callExpression(types.identifier(name), [args].concat(hasQuery ? [restArgs, types.stringLiteral(descriptor.id)] : [types.stringLiteral(descriptor.id)])));
        path.skip();
      }
    }

    return descriptor;
  }

  if (!types.isObjectExpression(args)) {
    return null;
  }

  args.properties.forEach(function (prop) {
    if (!types.isObjectProperty(prop)) {
      return;
    }

    var key = prop.key,
        value = prop.value;

    if (types.isIdentifier(key) && types.isStringLiteral(value)) {
      var val = value.value.trim();

      if (key.name === 'id') {
        descriptor.id = val;
      } else if (key.name === 'defaultMessage') {
        descriptor.defaultMessage = val;
        msgProp = prop;
      }
    }
  });

  if (translatedMessage != null && translatedMessage.messages) {
    descriptor.translatedMessage = translatedMessage == null ? void 0 : translatedMessage.messages;
  }

  if (descriptor.defaultMessage) {
    if (!descriptor.id) {
      descriptor.id = genId(descriptor.defaultMessage, translatedMessage == null ? void 0 : translatedMessage.scope); // insert id

      if (writeMode) {
        path.replaceWith(cloneNormalCallExpression(path, name, descriptor, msgProp, removeDefaultMessage));
        path.skip();
      }
    } else if (removeDefaultMessage) {
      path.replaceWith(cloneNormalCallExpression(path, name, descriptor, msgProp, true));
      path.skip();
    }

    return descriptor;
  }

  return null;
}

function checkRepeatedDescriptor(path, descriptor, storage) {
  var prev = storage.get(descriptor.id);

  if (prev != null && prev.length) {
    prev.forEach(function (item) {
      if (item.defaultMessage !== descriptor.defaultMessage) {
        var result = codeFrameColumns(path.hub.getCode(), path.node.loc);
        console.log('There are duplicate id, different messages:');
        console.log("ID:             " + item.id);
        console.log("defaultMessage: " + item.defaultMessage);
        console.log(result);
      }
    });
  }
}

function run(writeMode, options, storage) {
  var componentNames = ComponentNames.concat(options.componentNames || []);
  var functionNames = CalleeFunctionNames.concat(options.functionNames || []);
  var shortcut = 'shortcut' in options ? typeof options.shortcut === 'boolean' && options.shortcut ? ShortcutFunctionName : typeof options.shortcut === 'object' ? options.shortcut.functionName : '' : ShortcutFunctionName;
  return {
    pre: function pre() {
      if (storage && this.filename) {
        storage.set(this.filename, new Map());
      }
    },
    visitor: {
      JSXOpeningElement: function JSXOpeningElement(path, file) {
        var name = path.node.name;
        var filename = file.filename;

        if (filename && types.isJSXIdentifier(name) && componentNames.includes(name.name)) {
          var descriptor = createJSXMessageDescriptor(path, writeMode, options.removeDefaultMessage);

          if (descriptor && storage) {
            if (options.fileLoc) {
              descriptor.filename = filename;
              descriptor.loc = path.node.loc;
            }

            var container = storage.get(filename);
            checkRepeatedDescriptor(path, descriptor, container);

            if (container.has(descriptor.id)) {
              var _container$get;

              (_container$get = container.get(descriptor.id)) == null ? void 0 : _container$get.push(descriptor);
            } else {
              container.set(descriptor.id, [descriptor]);
            }
          }
        }
      },
      CallExpression: function CallExpression(path, file) {
        var callee = path.node.callee;
        var filename = file.filename;

        if (!filename) {
          return;
        }

        if (types.isIdentifier(callee) && functionNames.includes(callee.name) || types.isMemberExpression(callee) && types.isIdentifier(callee.property) && types.isIdentifier(callee.object) && functionNames.includes(callee.property.name)) {
          var descriptor = createFunctionMessageDescriptor(path, shortcut, writeMode, options.removeDefaultMessage);

          if (descriptor && storage) {
            if (options.fileLoc) {
              descriptor.filename = filename;
              descriptor.loc = path.node.loc;
            }

            var container = storage.get(filename);
            checkRepeatedDescriptor(path, descriptor, container);

            if (container.has(descriptor.id)) {
              var _container$get2;

              (_container$get2 = container.get(descriptor.id)) == null ? void 0 : _container$get2.push(descriptor);
            } else {
              container.set(descriptor.id, [descriptor]);
            }
          }
        }
      }
    }
  };
}

var plugin = /*#__PURE__*/declare(function (_, options) {
  return run(true, options);
});

var umiPlugin = (function (api) {
  api.describe({
    key: UmiPluginKey,
    config: {
      "default": undefined,
      schema: function schema(joi) {
        return joi.object({
          functionNames: joi.array().items(joi.string()),
          componentNames: joi.array().items(joi.string()),
          fileLoc: joi["boolean"](),
          removeDefaultMessage: joi["boolean"](),
          shortcut: [joi["boolean"](), joi.object({
            functionName: joi.string()
          })]
        });
      }
    }
  });
  api.modifyBabelOpts(function (babelOpts) {
    var _api$config;

    babelOpts.plugins.push([plugin, api == null ? void 0 : (_api$config = api.config) == null ? void 0 : _api$config[UmiPluginKey]]);
    return babelOpts;
  });
});

function _regeneratorRuntime() {
  /*! regenerator-runtime -- Copyright (c) 2014-present, Facebook, Inc. -- license (MIT): https://github.com/facebook/regenerator/blob/main/LICENSE */

  _regeneratorRuntime = function () {
    return exports;
  };

  var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";

  function define(obj, key, value) {
    return Object.defineProperty(obj, key, {
      value: value,
      enumerable: !0,
      configurable: !0,
      writable: !0
    }), obj[key];
  }

  try {
    define({}, "");
  } catch (err) {
    define = function (obj, key, value) {
      return obj[key] = value;
    };
  }

  function wrap(innerFn, outerFn, self, tryLocsList) {
    var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
    return generator._invoke = function (innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");

        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }

        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;

          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);

            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }

          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);

          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }

          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }(innerFn, self, context), generator;
  }

  function tryCatch(fn, obj, arg) {
    try {
      return {
        type: "normal",
        arg: fn.call(obj, arg)
      };
    } catch (err) {
      return {
        type: "throw",
        arg: err
      };
    }
  }

  exports.wrap = wrap;
  var ContinueSentinel = {};

  function Generator() {}

  function GeneratorFunction() {}

  function GeneratorFunctionPrototype() {}

  var IteratorPrototype = {};
  define(IteratorPrototype, iteratorSymbol, function () {
    return this;
  });
  var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
  NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
  var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);

  function defineIteratorMethods(prototype) {
    ["next", "throw", "return"].forEach(function (method) {
      define(prototype, method, function (arg) {
        return this._invoke(method, arg);
      });
    });
  }

  function AsyncIterator(generator, PromiseImpl) {
    function invoke(method, arg, resolve, reject) {
      var record = tryCatch(generator[method], generator, arg);

      if ("throw" !== record.type) {
        var result = record.arg,
            value = result.value;
        return value && "object" == typeof value && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
          invoke("next", value, resolve, reject);
        }, function (err) {
          invoke("throw", err, resolve, reject);
        }) : PromiseImpl.resolve(value).then(function (unwrapped) {
          result.value = unwrapped, resolve(result);
        }, function (error) {
          return invoke("throw", error, resolve, reject);
        });
      }

      reject(record.arg);
    }

    var previousPromise;

    this._invoke = function (method, arg) {
      function callInvokeWithMethodAndArg() {
        return new PromiseImpl(function (resolve, reject) {
          invoke(method, arg, resolve, reject);
        });
      }

      return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
    };
  }

  function maybeInvokeDelegate(delegate, context) {
    var method = delegate.iterator[context.method];

    if (undefined === method) {
      if (context.delegate = null, "throw" === context.method) {
        if (delegate.iterator.return && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
        context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
      }

      return ContinueSentinel;
    }

    var record = tryCatch(method, delegate.iterator, context.arg);
    if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
    var info = record.arg;
    return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
  }

  function pushTryEntry(locs) {
    var entry = {
      tryLoc: locs[0]
    };
    1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
  }

  function resetTryEntry(entry) {
    var record = entry.completion || {};
    record.type = "normal", delete record.arg, entry.completion = record;
  }

  function Context(tryLocsList) {
    this.tryEntries = [{
      tryLoc: "root"
    }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
  }

  function values(iterable) {
    if (iterable) {
      var iteratorMethod = iterable[iteratorSymbol];
      if (iteratorMethod) return iteratorMethod.call(iterable);
      if ("function" == typeof iterable.next) return iterable;

      if (!isNaN(iterable.length)) {
        var i = -1,
            next = function next() {
          for (; ++i < iterable.length;) if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;

          return next.value = undefined, next.done = !0, next;
        };

        return next.next = next;
      }
    }

    return {
      next: doneResult
    };
  }

  function doneResult() {
    return {
      value: undefined,
      done: !0
    };
  }

  return GeneratorFunction.prototype = GeneratorFunctionPrototype, define(Gp, "constructor", GeneratorFunctionPrototype), define(GeneratorFunctionPrototype, "constructor", GeneratorFunction), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
    var ctor = "function" == typeof genFun && genFun.constructor;
    return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
  }, exports.mark = function (genFun) {
    return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
  }, exports.awrap = function (arg) {
    return {
      __await: arg
    };
  }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
    return this;
  }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
    void 0 === PromiseImpl && (PromiseImpl = Promise);
    var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
    return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
      return result.done ? result.value : iter.next();
    });
  }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
    return this;
  }), define(Gp, "toString", function () {
    return "[object Generator]";
  }), exports.keys = function (object) {
    var keys = [];

    for (var key in object) keys.push(key);

    return keys.reverse(), function next() {
      for (; keys.length;) {
        var key = keys.pop();
        if (key in object) return next.value = key, next.done = !1, next;
      }

      return next.done = !0, next;
    };
  }, exports.values = values, Context.prototype = {
    constructor: Context,
    reset: function (skipTempReset) {
      if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
    },
    stop: function () {
      this.done = !0;
      var rootRecord = this.tryEntries[0].completion;
      if ("throw" === rootRecord.type) throw rootRecord.arg;
      return this.rval;
    },
    dispatchException: function (exception) {
      if (this.done) throw exception;
      var context = this;

      function handle(loc, caught) {
        return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
      }

      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i],
            record = entry.completion;
        if ("root" === entry.tryLoc) return handle("end");

        if (entry.tryLoc <= this.prev) {
          var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");

          if (hasCatch && hasFinally) {
            if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
          } else if (hasCatch) {
            if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
          } else {
            if (!hasFinally) throw new Error("try statement without catch or finally");
            if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
          }
        }
      }
    },
    abrupt: function (type, arg) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];

        if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
          var finallyEntry = entry;
          break;
        }
      }

      finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
      var record = finallyEntry ? finallyEntry.completion : {};
      return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
    },
    complete: function (record, afterLoc) {
      if ("throw" === record.type) throw record.arg;
      return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
    },
    finish: function (finallyLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
      }
    },
    catch: function (tryLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];

        if (entry.tryLoc === tryLoc) {
          var record = entry.completion;

          if ("throw" === record.type) {
            var thrown = record.arg;
            resetTryEntry(entry);
          }

          return thrown;
        }
      }

      throw new Error("illegal catch attempt");
    },
    delegateYield: function (iterable, resultName, nextLoc) {
      return this.delegate = {
        iterator: values(iterable),
        resultName: resultName,
        nextLoc: nextLoc
      }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
    }
  }, exports;
}

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends.apply(this, arguments);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);

  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }

  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var getScanOptions = function getScanOptions(root) {
  var options = {};
  var name = join(root, ScanConfigFilename);

  if (existsSync(name)) {
    Object.assign(options, JSON.parse(readFileSync(name).toString()));
  }

  return options;
};
var defaultScanConvertor = function defaultScanConvertor(data) {
  return Array.from(data.entries()).reduce(function (list, _ref) {
    var key = _ref[0],
        item = _ref[1];
    return list.concat({
      id: key,
      defaultMessage: item.defaultMessage,
      description: item.description
    });
  }, []);
};
var getTranslatedDir = function getTranslatedDir(root) {
  var filename = join(root, 'l10n.config.js');
  var content = existsSync(filename) ? require(filename) : null;
  var defaultDir = 'locales';
  var ret;

  if (!content) {
    ret = defaultDir;
    return join(root, ret);
  }

  try {
    var obj = content.middlewares;
    var match = obj.emit.match(/\?dest=([^?]+)/);
    ret = match ? match[1] : defaultDir;
  } catch (_) {
    ret = defaultDir;
  }

  return join(root, ret);
};

var globASync = /*#__PURE__*/promisify(glob);

var runPlguin = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(file, plugin) {
    return _regeneratorRuntime().wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return transformFileAsync(file, {
              presets: ['@babel/preset-env', '@babel/preset-react', '@babel/preset-typescript'],
              plugins: [['@babel/plugin-proposal-decorators', {
                legacy: true
              }], ['@babel/plugin-proposal-class-properties'], '@babel/plugin-syntax-dynamic-import', plugin]
            });

          case 2:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function runPlguin(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var pattern = /[a-z]{2}(?:-[A-Z]{2})?/;
var keysMap = /*#__PURE__*/new Map( /*#__PURE__*/['id', 'defaultMessage', 'filename', 'loc', 'translatedMessage'].map(function (item) {
  return [item, item];
}));
/**
 * 从列表中查找最合适的 descriptor
 * 含有翻译信息最多，如果一样但是内容不一样，给出警告
 * @param list
 */

var findDescriptor = function findDescriptor(list) {
  var locales = list.map(function (item) {
    return Object.keys(item).filter(function (key) {
      return !keysMap.has(key) && pattern.test(key);
    });
  });
  var max = Math.max.apply(Math, locales.map(function (item) {
    return item.length;
  }));
  var items = [];
  locales.forEach(function (item, index) {
    if (item.length === max) {
      items.push(list[index]);
    }
  });

  if (max === 0 || items.length === 1) {
    var _items$find;

    var target = (_items$find = items.find(function (item) {
      return item.translatedMessage;
    })) != null ? _items$find : items[0];
    return target;
  }

  console.warn('存在相同翻译元素但翻译内容不一样:');
  throw Error(JSON.stringify(items, null, 2));
};
/**
 * 提取翻译数据
 *
 * @param root 艮目录
 * @param source 待提取的文件
 * @param dist 输出文件名
 */


var scan = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee2(root, source, dist) {
    var options, internalTranslatedDataFilename, data, runScan, files, result, translatedData, _iterator, _step, _source, _iterator2, _step2, item, descriptor;

    return _regeneratorRuntime().wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (source === void 0) {
              source = 'src';
            }

            if (dist === void 0) {
              dist = I18nMessageFilename;
            }

            options = _extends({
              source: isAbsolute(source) ? source : join(root, source),
              dist: isAbsolute(dist) ? dist : join(root, dist),
              convertor: defaultScanConvertor
            }, getScanOptions(root));
            internalTranslatedDataFilename = join(dirname(options.dist), InternalTranslatedDataFilename);
            data = new Map();

            runScan = function runScan() {
              return run(false, options, data);
            };

            sync(dirname(options.dist));
            _context2.next = 9;
            return globASync(options.source + "/**/!(*.d).@(js|jsx|ts|tsx)");

          case 9:
            files = _context2.sent;
            _context2.next = 12;
            return Promise.all(files.map(function (file) {
              return runPlguin(file, runScan);
            }));

          case 12:
            result = new Map();
            translatedData = {};

            for (_iterator = _createForOfIteratorHelperLoose(data.values()); !(_step = _iterator()).done;) {
              _source = _step.value;

              for (_iterator2 = _createForOfIteratorHelperLoose(_source.values()); !(_step2 = _iterator2()).done;) {
                item = _step2.value;
                descriptor = findDescriptor(item);
                result.set(descriptor.id, descriptor);

                if (descriptor.translatedMessage) {
                  translatedData[descriptor.id] = descriptor.translatedMessage;
                }
              }
            }

            if (Object.keys(translatedData).length) {
              writeFileSync(internalTranslatedDataFilename, "module.exports = " + JSON.stringify(translatedData) + ";");
            }

            writeFileSync(options.dist, JSON.stringify(options.convertor(result)));

          case 17:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function scan(_x3, _x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();

var getExcelHeaders = function getExcelHeaders(locales) {
  return [{
    header: 'id',
    key: 'id'
  }].concat(Array.from(locales.values()).sort(function (a, b) {
    return b.localeCompare(a);
  }).map(function (item) {
    return {
      key: item,
      header: item,
      width: 50
    };
  }));
};

var mergePreviousExcelData = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(map, previousFilename) {
    var previousData, _iterator, _step, item, list;

    return _regeneratorRuntime().wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!existsSync(previousFilename)) {
              _context.next = 5;
              break;
            }

            _context.next = 3;
            return readExcelData(previousFilename);

          case 3:
            previousData = _context.sent;

            for (_iterator = _createForOfIteratorHelperLoose(previousData.values()); !(_step = _iterator()).done;) {
              item = _step.value;

              if (map.has(item.id)) {
                map.set(item.id, _extends({}, item, map.get(item.id)));
              }
            }

          case 5:
            list = Array.from(map.values());
            return _context.abrupt("return", list.sort(function (a, b) {
              return JSON.stringify(a).length - JSON.stringify(b).length;
            }));

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function mergePreviousExcelData(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();
var mergeInternalTranslatedData = function mergeInternalTranslatedData(map, translatedDataFilename) {
  if (existsSync(translatedDataFilename)) {
    var data = require(translatedDataFilename);

    Object.entries(data).forEach(function (_ref2) {
      var id = _ref2[0],
          item = _ref2[1];

      if (map.has(id)) {
        map.set(id, _extends({}, map.get(id), item));
      }
    });
  }

  return map;
};
var writeDataToExcel = /*#__PURE__*/function () {
  var _ref3 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee2(locales, data, filename) {
    var book, sheet;
    return _regeneratorRuntime().wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            book = new Excel.Workbook();
            sheet = book.addWorksheet('data');
            sheet.columns = getExcelHeaders(locales);
            sheet.addRows(data);
            _context2.next = 6;
            return book.xlsx.writeFile(filename);

          case 6:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function writeDataToExcel(_x3, _x4, _x5) {
    return _ref3.apply(this, arguments);
  };
}();
var readExcelData = /*#__PURE__*/function () {
  var _ref4 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee3(filename) {
    var book, map, sheet, headers;
    return _regeneratorRuntime().wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            book = new Excel.Workbook();
            _context3.next = 3;
            return book.xlsx.readFile(filename);

          case 3:
            // @ts-ignore
            map = new Map();
            sheet = book.worksheets[0];

            if (sheet !== null && sheet.rowCount > 0) {
              headers = sheet.getRow(1).values.slice(1);
              sheet.eachRow(function (row, index) {
                if (index === 1) {
                  return;
                }

                var values = row.values;
                var item = values.slice(1);
                map.set(item[0], _extends({
                  id: item[0]
                }, item.slice(1).reduce(function (map, val, index) {
                  var _extends2;

                  return _extends({}, map, (_extends2 = {}, _extends2[headers[index + 1]] = val.trim(), _extends2));
                }, {})));
              });
            }

            return _context3.abrupt("return", map);

          case 7:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function readExcelData(_x6) {
    return _ref4.apply(this, arguments);
  };
}();

var globASync$1 = /*#__PURE__*/promisify(glob);

var readData = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(file) {
    var content, locale, list;
    return _regeneratorRuntime().wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return promises.readFile(file);

          case 2:
            _context.next = 4;
            return _context.sent.toString();

          case 4:
            content = _context.sent;
            locale = basename(file, extname(file));
            list = Object.entries(JSON.parse(content)).reduce(function (ret, _ref2) {
              var id = _ref2[0],
                  defaultMessage = _ref2[1];
              return ret.concat({
                id: id,
                defaultMessage: defaultMessage
              });
            }, []);
            return _context.abrupt("return", {
              locale: locale,
              list: list
            });

          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function readData(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * 将翻译后的数据 提取到 excel,如果在目录下存在 excel , 则合并
 *
 * @param root 根目录
 * @param source 目标目录
 * @param dist 输出 Excel 文件名
 */


var pickL18nMessages = /*#__PURE__*/function () {
  var _ref3 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee2(root, source, dist) {
    var sourceDir, files, distFilename, internalTranslatedDataFilename, data, map, locales, list;
    return _regeneratorRuntime().wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (source === void 0) {
              source = /*#__PURE__*/dirname(I18nMessageFilename);
            }

            if (dist === void 0) {
              dist = ExcelFilename;
            }

            sourceDir = getTranslatedDir(root);
            _context2.next = 5;
            return globASync$1(sourceDir + "/*.json");

          case 5:
            files = _context2.sent;
            distFilename = isAbsolute(dist) ? dist : join(sourceDir, dist);
            internalTranslatedDataFilename = join(isAbsolute(source) ? source : join(root, source), InternalTranslatedDataFilename);
            _context2.next = 10;
            return Promise.all(files.map(function (file) {
              return readData(file);
            }));

          case 10:
            data = _context2.sent;
            map = new Map();
            locales = new Set();
            data.forEach(function (_ref4) {
              var locale = _ref4.locale,
                  list = _ref4.list;
              var realLocale = locale in LocaleMap ? LocaleMap[locale] : locale;
              locales.add(realLocale);
              list.forEach(function (item) {
                var _extends2;

                if (!map.has(item.id)) {
                  map.set(item.id, {
                    id: item.id
                  });
                }

                var target = map.get(item.id);
                map.set(item.id, _extends({}, target, (_extends2 = {}, _extends2[realLocale] = item.defaultMessage, _extends2)));
              });
            });
            _context2.next = 16;
            return mergePreviousExcelData(mergeInternalTranslatedData(map, internalTranslatedDataFilename), distFilename);

          case 16:
            list = _context2.sent;
            _context2.next = 19;
            return writeDataToExcel(locales, list, distFilename);

          case 19:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function pickL18nMessages(_x2, _x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}();

var insertImport = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(filename, locale, name) {
    var content;
    return _regeneratorRuntime().wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return promises.readFile(filename);

          case 2:
            _context.next = 4;
            return _context.sent.toString();

          case 4:
            content = _context.sent;

            if (content && !content.includes(name)) {
              content = content.replace(/\s*export\s+default/, function (match) {
                return content.includes(name) ? match : "\nimport " + name + " from \"./" + locale + "/" + name + "\";" + match;
              });
              content = content.replace(/(,)?(\s*};?\s*$)/, function (_, $1, $2) {
                return ($1 || ',') + "\n  ..." + name + "," + $2;
              });
            } else if (!content) {
              content = "import " + name + " from \"./" + locale + "/" + name + "\";\n\nexport default {\n  ... " + name + "\n};\n    ";
            }

            _context.next = 8;
            return promises.writeFile(filename, content);

          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function insertImport(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * 将 Excel 文件里的翻译数据分发到代码里
 *
 * @param root 根目录
 * @param dist 目标路径 , 参照 umi locales
 * @param excel Excel 文件
 */


var distribute = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee3(root, dist, excel) {
    var sourceDir, distDir, excelFilename, data, list, keys, locales, dirs, collections;
    return _regeneratorRuntime().wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            if (dist === void 0) {
              dist = 'src/locales';
            }

            if (excel === void 0) {
              excel = ExcelFilename;
            }

            sourceDir = getTranslatedDir(root);
            distDir = isAbsolute(dist) ? dist : join(root, dist);
            excelFilename = join(sourceDir, excel);
            _context3.next = 7;
            return readExcelData(excelFilename);

          case 7:
            data = _context3.sent;

            if (!(data.size === 0)) {
              _context3.next = 10;
              break;
            }

            return _context3.abrupt("return");

          case 10:
            list = Array.from(data.values());
            keys = list.map(Object.keys).sort(function (a, b) {
              return b.length - a.length;
            });
            locales = keys[0].filter(function (item) {
              return item !== 'id';
            });
            dirs = locales.map(function (locale) {
              return join(distDir, locale);
            });
            collections = locales.map(function (locale) {
              return {
                locale: locale,
                data: list.reduce(function (map, item) {
                  var _extends2;

                  return _extends({}, map, (_extends2 = {}, _extends2[item.id] = item[locale], _extends2));
                }, {})
              };
            });
            dirs.map(sync);
            dirs.forEach(function (dir) {
              sync(dir);

              if (!existsSync(dir + ".ts")) {
                writeFileSync(dir + ".ts", '');
              }
            });
            _context3.next = 19;
            return Promise.all(collections.map( /*#__PURE__*/function () {
              var _ref4 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee2(_ref3) {
                var locale, data, content, targetFilename;
                return _regeneratorRuntime().wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        locale = _ref3.locale, data = _ref3.data;
                        content = "export default " + JSON.stringify(data, null, 2) + ";";
                        targetFilename = join(distDir, locale, DistributionName + ".ts");
                        _context2.next = 5;
                        return Promise.all([promises.writeFile(targetFilename, content), insertImport(join(distDir, locale + ".ts"), locale, DistributionName)]);

                      case 5:
                      case "end":
                        return _context2.stop();
                    }
                  }
                }, _callee2);
              }));

              return function (_x7) {
                return _ref4.apply(this, arguments);
              };
            }()));

          case 19:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function distribute(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();

/**
 * 将自动提取到的数据 提取到 excel,如果在目录下存在 excel , 则合并
 *
 * @param root  根目录
 * @param source  目标路径
 * @param locale 默认语言
 */

var transform = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(root, source, excelFilename, locale) {
    var l18nSourceDir, content, internalTranslatedDataFilename, distFilename, map, locales, list;
    return _regeneratorRuntime().wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (source === void 0) {
              source = I18nMessageFilename;
            }

            if (excelFilename === void 0) {
              excelFilename = ExcelFilename;
            }

            if (locale === void 0) {
              locale = 'zh-CN';
            }

            source = isAbsolute(source) ? source : join(root, source);
            l18nSourceDir = getTranslatedDir(root);
            content = JSON.parse(readFileSync(source).toString());
            internalTranslatedDataFilename = join(dirname(source), InternalTranslatedDataFilename);
            distFilename = isAbsolute(excelFilename) ? excelFilename : join(l18nSourceDir, ExcelFilename);
            map = new Map();
            locales = new Set(Object.values(LocaleMap));

            if (!isAbsolute(excelFilename)) {
              sync(l18nSourceDir);
            }

            content.forEach(function (item) {
              var _map$set;

              map.set(item.id, (_map$set = {
                id: item.id
              }, _map$set[locale] = item.defaultMessage, _map$set));
            });
            mergeInternalTranslatedData(map, internalTranslatedDataFilename);
            _context.next = 15;
            return mergePreviousExcelData(map, distFilename);

          case 15:
            list = _context.sent;
            _context.next = 18;
            return writeDataToExcel(locales, list, distFilename);

          case 18:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function transform(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
}();

var execAsync = /*#__PURE__*/promisify(exec);
/**
 * 使用 atool-ll10n 翻译
 */

var translate = /*#__PURE__*/(function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(root) {
    return _regeneratorRuntime().wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return execAsync('npx atool-l10n', {
              cwd: root
            });

          case 2:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();

export default umiPlugin;
export { plugin as BabelPluginUmiMesIntl, distribute, genId as id, pickL18nMessages, scan, transform, translate };
//# sourceMappingURL=mes-babel-plugin-intl.esm.js.map
